﻿using MesETL.App.Helpers;
using Microsoft.Extensions.Logging;

namespace MesETL.App.Services.ErrorRecorder;

public class ErrorRecorder
{
    protected ILogger Logger;

    /// <summary>
    /// 当次执行标识
    /// </summary>
    public static readonly string UID = DateTime.Now.ToString("yyyy-MM-dd HH-mm-ss");

    public ErrorRecorder(ILogger logger)
    {
        Logger = logger;
    }
    
    
     public static async Task LogErrorRecordAsync(string outputDir, DataRecord record, Exception exception)
     {
         if(!Directory.Exists(outputDir))
             Directory.CreateDirectory(outputDir);
         var content = $"""
                        ### {exception.Message}
                        {record.RawField}
                        """;
         var path = Path.Combine(outputDir, $"{record.TableName}.errlog");
         await File.AppendAllTextAsync(path, content);
     }
     
     public static async Task LogErrorRecordAsync(string outputDir, IEnumerable<DataRecord> records, Exception exception)
     {
         if(!Directory.Exists(outputDir))
             Directory.CreateDirectory(outputDir);
         var tableMapping = new Dictionary<string, Tuple<List<DataRecord>, StreamWriter>>();
         foreach (var record in records)
         {
             tableMapping.AddOrUpdate(record.TableName,
                 Tuple.Create((List<DataRecord>) [record], new StreamWriter(File.OpenRead(record.TableName))),
                 (_, tuple) =>
                 {
                     tuple.Item1.Add(record);
                     return tuple;
                 });
         }

         var maxParallelism = 5;
         for (var i = 0; i < tableMapping.Count; i+=maxParallelism)
         {
             await Parallel.ForEachAsync(tableMapping.Take(maxParallelism), async (pair, token) =>
             {
                 var (records, writer) = pair.Value;
                 foreach (var record in records)
                 {
                     var content =
                         $"""
                          ### {exception.Message}
                          {record.RawField}
                          """;
                     await writer.WriteLineAsync(content);
                     if (token.IsCancellationRequested)
                         break;
                 }
                 await writer.DisposeAsync();
             });
         }
     }
     
     public void ClearErrorRecords(string dir)
     {
         Logger.LogInformation("***** Clear error records *****");
         foreach (var file in Directory.GetFiles(dir, "*.errlog", SearchOption.AllDirectories))
         {
             File.Delete(file);
         }
     }
}